@@ -36,4 +36,4 @@ if __name__ == '__main__':
     years = set(date.split('-')[0] for date in root.searcher.terms('date'))
     root.searcher.filters['year'] = dict((year, engine.Query.prefix('date', year).filter()) for year in years)
     # start with pretty-printing
-    server.start(root, config={'global': {'tools.json.indent': 2}})
+    server.start(root, config={'global': {'tools.json_out.indent': 2}})
