@@ -9,9 +9,11 @@ import warnings
 import lucene
 warnings.simplefilter('default', DeprecationWarning)
 
-from .queries import Query, SortField
+from .queries import Query, SortField, TermsFilter
 from .documents import Document, Field, FormatField, NestedField, NumericField, DateTimeField
-from .indexers import TokenFilter, Analyzer, IndexSearcher, MultiSearcher, IndexWriter, Indexer
+from .indexers import TokenFilter, Analyzer, IndexSearcher, MultiSearcher, IndexWriter, Indexer, ParallelIndexer
 from .spatial import PointField, PolygonField
 
 assert lucene.VERSION >= '3.1'
+if lucene.VERSION < '3.2':
+    warnings.warn('Support for lucene 3.1 will be removed in the next release.', DeprecationWarning)
