@@ -10,8 +10,9 @@ import re
 import itertools, operator
 import contextlib
 import abc, collections
+import warnings
 import lucene
-from .queries import Query, Collector, SortField, Highlighter, FastVectorHighlighter, SpellChecker, SpellParser
+from .queries import Query, Collector, TermsFilter, SortField, Highlighter, FastVectorHighlighter, SpellChecker, SpellParser
 from .documents import Field, Document, Hits
 from .spatial import DistanceComparator
 
@@ -28,7 +29,7 @@ class closing(set):
     "Manage lifespan of registered objects, similar to contextlib.closing."
     def __del__(self):
         for obj in self:
-            obj.close()
+            getattr(obj, 'decRef', obj.close)()
     def analyzer(self, analyzer, version=None):
         if analyzer is None:
             analyzer = lucene.StandardAnalyzer(version or lucene.Version.values()[-1])
@@ -69,23 +70,14 @@ def copy(commit, dest):
                 if not os.path.samefile(*paths):
                     raise
 
-class TokenFilter(lucene.PythonTokenFilter):
-    """Create an iterable lucene TokenFilter from a TokenStream.
-    Subclass and override :meth:`incrementToken`.
-    Attributes are cached as properties to create a Token interface.
-    """
-    def __init__(self, input):
-        lucene.PythonTokenFilter.__init__(self, input)
-        self.input = input
+class TokenStream(lucene.TokenStream):
+    "TokenStream mixin with support for iteration and attributes cached as properties."
     def __iter__(self):
         return self
     def next(self):
         if self.incrementToken():
             return self
         raise StopIteration
-    def incrementToken(self):
-        "Advance to next token and return whether the stream is not empty."
-        return self.input.incrementToken()
     def __getattr__(self, name):
         cls = getattr(lucene, name + 'Attribute').class_
         attr = self.getAttribute(cls) if self.hasAttribute(cls) else self.addAttribute(cls)
@@ -128,6 +120,21 @@ class TokenFilter(lucene.PythonTokenFilter):
     def type(self, text):
         self.Type.setType(text)
 
+class TokenFilter(lucene.PythonTokenFilter, TokenStream):
+    """Create an iterable lucene TokenFilter from a TokenStream.
+    Subclass and override :meth:`incrementToken` or :meth:`setattrs`.
+    """
+    def __init__(self, input):
+        lucene.PythonTokenFilter.__init__(self, input)
+        self.input = input
+    def incrementToken(self):
+        "Advance to next token and return whether the stream is not empty."
+        result = self.input.incrementToken()
+        self.setattrs()
+        return result
+    def setattrs(self):
+        "Customize current token."
+
 class Analyzer(lucene.PythonAnalyzer):
     """Return a lucene Analyzer which chains together a tokenizer and filters.
     
@@ -208,14 +215,15 @@ class IndexReader(object):
     def segments(self):
         "segment filenames with document counts"
         return dict((lucene.SegmentReader.cast_(reader).segmentName, reader.numDocs()) for reader in self.sequentialSubReaders)
-    def copy(self, dest, query=None, exclude=None, optimize=False):
+    def copy(self, dest, query=None, exclude=None, optimize=False, merge=0):
         """Copy the index to the destination directory.
         Optimized to use hard links if the destination is a file system path.
         
         :param dest: destination directory path or lucene Directory
         :param query: optional lucene Query to select documents
         :param exclude: optional lucene Query to exclude documents
-        :param optimize: optionally optimize destination index
+        :param merge: optionally merge into maximum number of segments
+        :param optimize: .. deprecated:: 1.2 use **merge** param instead
         """
         copy(self.indexCommit, dest)
         with contextlib.closing(IndexWriter(dest)) as writer:
@@ -226,15 +234,23 @@ class IndexReader(object):
             writer.commit()
             writer.expungeDeletes()
             if optimize:
-                writer.optimize(optimize)
+                warnings.warn("Use merge=int instead of optimize parameter.", DeprecationWarning)
+                merge = int(optimize)
+            if merge:
+                writer.optimize(merge)
             return len(writer)
     def count(self, name, value):
         "Return number of documents with given term."
         return self.docFreq(lucene.Term(name, value))
-    def names(self, option='all'):
-        "Return field names, given option description."
-        option = getattr(self.FieldOption, option.upper())
-        return list(self.getFieldNames(option))
+    def names(self, option='all', **attrs):
+        """Return field names, given option description.
+        
+        .. versionchanged:: 1.2 lucene 3.6 requires FieldInfo filter attributes instead of option
+        """
+        if hasattr(lucene.IndexReader, 'getFieldNames'):
+            return list(self.getFieldNames(getattr(self.FieldOption, option.upper())))
+        fieldinfos = lucene.ReaderUtil.getMergedFieldInfos(self.indexReader).iterator()
+        return [fieldinfo.name for fieldinfo in fieldinfos if all(getattr(fieldinfo, name) == attrs[name] for name in attrs)]
     def terms(self, name, value='', stop=None, counts=False, **fuzzy):
         """Generate a slice of term values, optionally with frequency counts.
         Supports a range of terms, wildcard terms, or fuzzy terms.
@@ -361,9 +377,11 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
     """
     def __init__(self, directory, analyzer=None):
         self.shared = closing()
-        lucene.IndexSearcher.__init__(self, self.shared.directory(directory))
+        lucene.IndexSearcher.__init__(self, self.shared.reader(directory))
+        self.owned = closing([self.indexReader])
         self.analyzer = self.shared.analyzer(analyzer)
         self.filters, self.sorters, self.spellcheckers = {}, {}, {}
+        self.termsfilters = set()
     @classmethod
     def load(cls, directory, analyzer=None):
         "Open `IndexSearcher`_ with a lucene RAMDirectory, loading index into memory."
@@ -376,6 +394,7 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
             self.close()
     def reopen(self, filters=False, sorters=False, spellcheckers=False):
         """Return current `IndexSearcher`_, only creating a new one if necessary.
+        Any registered :attr:`termsfilters` are also refreshed.
         
         :param filters: refresh cached facet :attr:`filters`
         :param sorters: refresh cached :attr:`sorters` with associated parsers
@@ -384,8 +403,11 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
         if self.current:
             return self
         other = type(self)(self.indexReader.reopen(), self.analyzer)
-        other.shared, other.owned = self.shared, closing([other.indexReader])
+        other.decRef()
+        other.shared = self.shared
         other.filters.update((key, value if isinstance(value, lucene.Filter) else dict(value)) for key, value in self.filters.items())
+        for termsfilter in self.termsfilters:
+            termsfilter.refresh(other)
         if filters:
             other.facets(Query.any(), *other.filters)
         other.sorters = dict((name, SortField(sorter.field, sorter.typename, sorter.parser)) for name, sorter in self.sorters.items())
@@ -412,7 +434,7 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
     def highlighter(self, query, field, **kwargs):
         "Return `Highlighter`_ or if applicable `FastVectorHighlighter`_ specific to searcher and query."
         query = self.parse(query, field=field)
-        vector = field in self.names('termvector_with_position_offset')
+        vector = field in self.names('termvector_with_position_offset', storeTermVector=True)
         return (FastVectorHighlighter if vector else Highlighter)(self, query, field, **kwargs)
     def count(self, *query, **options):
         """Return number of hits for given query or term.
@@ -445,7 +467,7 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
         if count is None:
             collector = Collector()
         else:
-            count, inorder = min(count, self.maxDoc()), not weight.scoresDocsOutOfOrder()
+            count, inorder = min(count, self.maxDoc() or 1), not weight.scoresDocsOutOfOrder()
             if sort is None:
                 collector = lucene.TopScoreDocCollector.create(count, inorder)
             else:
@@ -562,8 +584,8 @@ class MultiSearcher(IndexSearcher):
     def __init__(self, reader, analyzer=None):
         shared = closing()
         if not lucene.MultiReader.instance_(reader):
-            reader = lucene.MultiReader(list(map(shared.reader, reader)), True)
-            self.owned = closing([reader])
+            reader = lucene.MultiReader(list(map(shared.reader, reader)))
+            owned = closing([reader]) # decRef when finished
         IndexSearcher.__init__(self, reader, analyzer)
         self.shared.update(shared)
         shared.clear()
@@ -584,17 +606,20 @@ class IndexWriter(lucene.IndexWriter):
     :param mode: file mode (rwa), except updating (+) is implied
     :param analyzer: lucene Analyzer, default StandardAnalyzer
     :param version: lucene Version argument passed to IndexWriterConfig or StandardAnalyzer, default is latest
+    :param attrs: additional attributes to set on IndexWriterConfig
     """
     __len__ = lucene.IndexWriter.numDocs
     __del__ = IndexSearcher.__dict__['__del__']
     parse = IndexSearcher.__dict__['parse']
-    def __init__(self, directory=None, mode='a', analyzer=None, version=None):
+    def __init__(self, directory=None, mode='a', analyzer=None, version=None, **attrs):
         self.shared = closing()
         if version is None:
             version = lucene.Version.values()[-1]
         config = lucene.IndexWriterConfig(version, self.shared.analyzer(analyzer, version))
         config.openMode = lucene.IndexWriterConfig.OpenMode.values()['wra'.index(mode)]
-        config.indexDeletionPolicy = self.policy = lucene.SnapshotDeletionPolicy(lucene.KeepOnlyLastCommitDeletionPolicy())
+        for name, value in attrs.items():
+            setattr(config, name, value)
+        config.indexDeletionPolicy = self.policy = lucene.SnapshotDeletionPolicy(config.indexDeletionPolicy)
         lucene.IndexWriter.__init__(self, self.shared.directory(directory), config)
         self.fields = {}
     def set(self, name, cls=Field, **params):
@@ -654,16 +679,11 @@ class Indexer(IndexWriter):
     
     :param nrt: optionally use a near real-time searcher
     """
-    def __init__(self, directory=None, mode='a', analyzer=None, version=None, nrt=False):
-        IndexWriter.__init__(self, directory, mode, analyzer, version)
+    def __init__(self, directory=None, mode='a', analyzer=None, version=None, nrt=False, **attrs):
+        IndexWriter.__init__(self, directory, mode, analyzer, version, **attrs)
         IndexWriter.commit(self)
         self.nrt = nrt
-        if nrt:
-            reader = lucene.IndexReader.open(self, True)
-            self.indexSearcher = IndexSearcher(reader, self.analyzer)
-            self.indexSearcher.owned = closing([reader])
-        else:
-            self.indexSearcher = IndexSearcher(self.directory, self.analyzer)
+        self.indexSearcher = IndexSearcher(self if nrt else self.directory, self.analyzer)
     def __getattr__(self, name):
         if name == 'indexSearcher':
             raise AttributeError(name)
@@ -677,17 +697,58 @@ class Indexer(IndexWriter):
     def refresh(self, **caches):
         "Store refreshed searcher with :meth:`IndexSearcher.reopen` caches."
         self.indexSearcher = self.indexSearcher.reopen(**caches)
-    def commit(self, expunge=False, optimize=False, **caches):
+    def commit(self, expunge=False, optimize=False, merge=False, **caches):
         """Commit writes and :meth:`refresh` searcher.
         
-        :param expunge: expunge deletes
-        :param optimize: optimize index, optionally supply number of segments
+        :param merge: merge segments with deletes, or optionally specify maximum number of segments
+        :param expunge,optimize: .. deprecated:: 1.2 use **merge** param instead
         """
         IndexWriter.commit(self)
         if expunge:
+            warnings.warn("Use merge=True instead of expunge parameter.", DeprecationWarning)
             self.expungeDeletes()
             IndexWriter.commit(self)
         if optimize:
-            self.optimize(optimize)
+            warnings.warn("Use merge=int instead of optimize parameter.", DeprecationWarning)
+            merge = int(optimize)
+        if merge:
+            if isinstance(merge, bool):
+                self.expungeDeletes()
+            else:
+                self.optimize(merge)
             IndexWriter.commit(self)
         self.refresh(**caches)
+
+class ParallelIndexer(Indexer):
+    """Indexer which tracks a unique identifying field.
+    Handles atomic updates of rapidly changing fields, managing :attr:`termsfilters`.
+    """
+    def __init__(self, field, *args, **kwargs):
+        Indexer.__init__(self, *args, **kwargs)
+        self.field = field
+        self.set(field, index=True, omitNorms=True)
+        self.termsfilters = {}
+    def termsfilter(self, filter, *others):
+        "Return `TermsFilter`_ synced to given filter and optionally associated with other indexers."
+        terms = self.sorter(self.field).terms(filter, *self.sequentialSubReaders)
+        termsfilter = self.termsfilters[filter] = TermsFilter(self.field, terms)
+        for other in others:
+            termsfilter.refresh(other)
+            other.termsfilters.add(termsfilter)
+        return termsfilter
+    def update(self, value, document=(), **terms):
+        "Atomically update document based on unique field."
+        terms[self.field] = value
+        self.updateDocument(lucene.Term(self.field, value), self.document(document, **terms))
+    def refresh(self, **caches):
+        "Store refreshed searcher and synchronize :attr:`termsfilters`."
+        sorter, segments = self.sorter(self.field), self.segments
+        searcher = self.indexSearcher.reopen(**caches)
+        readers = [reader for reader in searcher.sequentialSubReaders if lucene.SegmentReader.cast_(reader).segmentName not in segments]
+        terms = list(itertools.chain.from_iterable(IndexReader(reader).terms(self.field) for reader in readers))
+        for filter, termsfilter in self.termsfilters.items():
+            if terms:
+                termsfilter.update(terms, op='andNot', cache=not self.nrt)
+            if readers:
+                termsfilter.update(sorter.terms(filter, *readers), cache=not self.nrt)
+        self.indexSearcher = searcher
