@@ -6,14 +6,9 @@ from future_builtins import filter, map
 import itertools
 import bisect
 import heapq
+import threading
 import lucene
 
-class ArrayList(lucene.ArrayList):
-    def __init__(self, values=()):
-        lucene.ArrayList.__init__(self)
-        for value in values:
-            self.add(value)
-
 class Query(object):
     """Inherited lucene Query, with dynamic base class acquisition.
     Uses class methods and operator overloading for convenient query construction.
@@ -24,7 +19,15 @@ class Query(object):
         base.__init__(self, *args)
     def filter(self, cache=True):
         "Return lucene CachingWrapperFilter, optionally just QueryWrapperFilter."
-        filter = lucene.QueryWrapperFilter(self)
+        if isinstance(self, lucene.PrefixQuery):
+            filter = lucene.PrefixFilter(self.getPrefix())
+        elif isinstance(self, lucene.TermRangeQuery):
+            filter = lucene.TermRangeFilter(self.field, self.lowerTerm, self.upperTerm, self.includesLower(), self.includesUpper())
+        elif isinstance(self, lucene.TermQuery):
+            filter = lucene.TermsFilter()
+            filter.addTerm(self.getTerm())
+        else:
+            filter = lucene.QueryWrapperFilter(self)
         return lucene.CachingWrapperFilter(filter) if cache else filter
     def terms(self):
         "Generate set of query term items."
@@ -58,7 +61,7 @@ class Query(object):
     @classmethod
     def disjunct(cls, multiplier, *queries, **terms):
         "Return lucene DisjunctionMaxQuery from queries and terms."
-        self = cls(lucene.DisjunctionMaxQuery, ArrayList(queries), multiplier)
+        self = cls(lucene.DisjunctionMaxQuery, lucene.Arrays.asList(queries), multiplier)
         for name, values in terms.items():
             for value in ([values] if isinstance(values, basestring) else values):
                 self.add(cls.term(name, value))
@@ -146,6 +149,10 @@ class BooleanQuery(Query):
 
 class SpanQuery(Query):
     "Inherited lucene SpanQuery with additional span constructors."
+    def filter(self, cache=True):
+        "Return lucene CachingSpanFilter, optionally just SpanQueryFilter."
+        filter = lucene.SpanQueryFilter(self)
+        return lucene.CachingSpanFilter(filter) if cache else filter
     def __getitem__(self, slc):
         start, stop, step = slc.indices(lucene.Integer.MAX_VALUE)
         assert step == 1, 'slice step is not supported'
@@ -169,7 +176,7 @@ class SpanQuery(Query):
     def payload(self, *values):
         "Return lucene SpanPayloadCheckQuery from payload values."
         base = lucene.SpanNearPayloadCheckQuery if lucene.SpanNearQuery.instance_(self) else lucene.SpanPayloadCheckQuery
-        return SpanQuery(base, self, ArrayList(map(lucene.JArray_byte, values)))
+        return SpanQuery(base, self, lucene.Arrays.asList(list(map(lucene.JArray_byte, values))))
 
 class Collector(lucene.PythonCollector):
     "Collect all ids and scores efficiently."
@@ -190,9 +197,63 @@ class Collector(lucene.PythonCollector):
         ids.sort(key=key, reverse=reverse)
         return ids, list(map(self.scores.__getitem__, ids))
 
+class TermsFilter(lucene.CachingWrapperFilter):
+    """Caching filter based on a unique field and set of matching values.
+    Optimized for many terms and docs, with support for incremental updates.
+    Suitable for searching external metadata associated with indexed identifiers.
+    Call :meth:`refresh` to cache a new (or reopened) reader.
+    
+    :param field: field name
+    :param values: initial term values, synchronized with the cached filters
+    """
+    ops = {'or': 'update', 'and': 'intersection_update', 'andNot': 'difference_update'}
+    def __init__(self, field, values=()):
+        assert hasattr(lucene, 'FixedBitSet'), 'requires FixedBitSet introduced in lucene 3.4'
+        lucene.CachingWrapperFilter.__init__(self, lucene.TermsFilter())
+        self.field = field
+        self.values = set(values)
+        self.readers = set()
+        self.lock = threading.Lock()
+    def filter(self, values, cache=True):
+        "Return lucene TermsFilter, optionally using the FieldCache."
+        if cache:
+            return lucene.FieldCacheTermsFilter(self.field, tuple(values))
+        filter, term = lucene.TermsFilter(), lucene.Term(self.field)
+        for value in values:
+            filter.addTerm(term.createTerm(value))
+        return filter
+    def apply(self, filter, op, readers):
+        for reader in readers:
+            bitset = lucene.FixedBitSet.cast_(self.getDocIdSet(reader))
+            getattr(bitset, op)(filter.getDocIdSet(reader).iterator())
+    def update(self, values, op='or', cache=True):
+        """Update allowed values and corresponding cached bitsets.
+        
+        :param values: additional term values
+        :param op: set operation used to combine terms and docs
+        :param cache: optionally cache all term values using FieldCache
+        """
+        values = tuple(values)
+        filter = self.filter(values, cache)
+        with self.lock:
+            getattr(self.values, self.ops[op])(values)
+            self.apply(filter, op, self.readers)
+    def refresh(self, reader):
+        "Refresh cached bitsets of current values for new segments of top-level reader."
+        readers = set(reader.sequentialSubReaders)
+        with self.lock:
+            self.apply(self.filter(self.values), 'or', readers - self.readers)
+            self.readers = set(reader for reader in readers | self.readers if reader.refCount)
+    def add(self, *values):
+        "Add a few term values."
+        self.update(values, cache=False)
+    def discard(self, *values):
+        "Discard a few term values."
+        self.update(values, op='andNot', cache=False)
+
 class SortField(lucene.SortField):
     """Inherited lucene SortField used for caching FieldCache parsers.
-        
+    
     :param name: field name
     :param type: type object or name compatible with SortField constants
     :param parser: lucene FieldCache.Parser or callable applied to field values
@@ -207,14 +268,15 @@ class SortField(lucene.SortField):
             namespace = {'parse' + type: staticmethod(parser)}
             parser = object.__class__(base.__name__, (base,), namespace)()
         lucene.SortField.__init__(self, name, parser, reverse)
-    def comparator(self, reader):
-        "Return indexed values from default FieldCache using the given reader."
+    def array(self, reader):
         method = getattr(lucene.FieldCache.DEFAULT, 'get{0}s'.format(self.typename))
-        args = [self.parser] * bool(self.parser)
+        return method(reader, self.field, *[self.parser][:bool(self.parser)])
+    def comparator(self, reader):
+        "Return indexed values from default FieldCache using the given top-level reader."
         readers = reader.sequentialSubReaders
         if lucene.MultiReader.instance_(reader):
             readers = itertools.chain.from_iterable(reader.sequentialSubReaders for reader in readers)
-        arrays = [method(reader, self.field, *args) for reader in readers]
+        arrays = list(map(self.array, readers))
         if len(arrays) <= 1:
             return arrays[0]
         cls, = set(map(type, arrays))
@@ -223,6 +285,19 @@ class SortField(lucene.SortField):
             lucene.System.arraycopy(array, 0, result, index, len(array))
             index += len(array)
         return result
+    def filter(self, start, stop, lower=True, upper=False):
+        "Return lucene FieldCacheRangeFilter based on field and type."
+        method = getattr(lucene.FieldCacheRangeFilter, 'new{0}Range'.format(self.typename))
+        return method(self.field, self.parser, start, stop, lower, upper)
+    def terms(self, filter, *readers):
+        "Generate field cache terms from docs which match filter from all segments."
+        for reader in readers:
+            array, it = self.array(reader), filter.getDocIdSet(reader).iterator()
+            try:
+                while True:
+                    yield array[it.nextDoc()]
+            except IndexError:
+                pass
 
 class Highlighter(lucene.Highlighter):
     """Inherited lucene Highlighter with stored analysis options.
