@@ -2,13 +2,12 @@
 Restful json `CherryPy <http://cherrypy.org/>`_ server.
 
 The server script mounts a `WebSearcher`_ (read_only) or `WebIndexer`_ root.
-Standard `CherryPy configuration <http://www.cherrypy.org/wiki/ConfigAPI>`_ applies,
+Standard `CherryPy configuration <http://docs.cherrypy.org/stable/concepts/config.html>`_ applies,
 and the provided `custom tools <#tools>`_ are also configurable.
 All request and response bodies are `application/json values <http://tools.ietf.org/html/rfc4627.html#section-2.1>`_.
 
 WebSearcher exposes resources for an IndexSearcher.
 In addition to search requests, it provides access to term and document information in the index.
-Note Lucene doc ids are ephemeral;  they should only be used across requests for the same index version.
 
  * :meth:`/ <WebSearcher.index>`
  * :meth:`/search <WebSearcher.search>`
@@ -20,8 +19,8 @@ WebIndexer extends WebSearcher, exposing additional resources and methods for an
 Single documents may be added, deleted, or replaced by a unique indexed field.
 Multiples documents may also be added or deleted by query at once.
 By default changes are not visible until the update resource is called to commit a new index version.
-If a near real-time Indexer is used (an experimental feature in Lucene), then changes are instantly searchable.
-In such cases a commit still hasn't occurred;  the index based :meth:`last-modified header <validate>` shouldn't be used for caching.
+If a near real-time Indexer is used, then changes are instantly searchable.
+In such cases a commit still hasn't occurred, and the index based :meth:`last-modified header <validate>` shouldn't be used for caching.
 
  * :meth:`/ <WebIndexer.index>`
  * :meth:`/search <WebIndexer.search>`
@@ -43,7 +42,8 @@ CherryPy and Lucene VM integration issues:
 from future_builtins import filter, map
 import re
 import time
-import httplib
+import uuid
+import socket, httplib
 import heapq
 import collections
 import itertools, operator
@@ -57,23 +57,22 @@ except ImportError:
 import lucene
 import cherrypy
 try:
-    from . import engine
+    from . import engine, client
 except ValueError:
-    import engine
+    import engine, client
 
 def tool(hook):
     "Return decorator to register tool at given hook point."
     def decorator(func):
-        setattr(cherrypy.tools, func.__name__.rstrip('_'), cherrypy.Tool(hook, func))
+        setattr(cherrypy.tools, func.__name__, cherrypy.Tool(hook, func))
         return func
     return decorator
 
 @tool('before_handler')
-def json_(indent=None, content_type='application/json', process_body=None):
-    """Handle request bodies and responses in json format.
+def json_in(content_type='application/json', process_body=None):
+    """Handle request bodies in json format.
     
-    :param indent: indentation level for pretty printing
-    :param content_type: request media type and response content-type header
+    :param content_type: request media type
     :param process_body: optional function to process body into request.params
     """
     request = cherrypy.serving.request
@@ -84,18 +83,25 @@ def json_(indent=None, content_type='application/json', process_body=None):
         if process_body is not None:
             with HTTPError(httplib.BAD_REQUEST, TypeError):
                 request.params.update(process_body(request.json))
-    elif media_type is not None:
-        message = "Received Content-Type header {0}; only {1} is supported.".format(media_type, content_type)
+    elif media_type:
+        message = "Expected an entity of content type {0}".format(content_type)
         raise cherrypy.HTTPError(httplib.UNSUPPORTED_MEDIA_TYPE, message)
+
+@tool('before_handler')
+def json_out(content_type='application/json', indent=None):
+    """Handle responses in json format.
+    
+    :param content_type: response content-type header
+    :param indent: indentation level for pretty printing
+    """
+    request = cherrypy.serving.request
+    request._json_inner_handler = request.handler
     headers = cherrypy.response.headers
-    handler = request.handler
-    def json_handler(*args, **kwargs):
-        body = handler(*args, **kwargs)
-        if headers['content-type'].startswith('text/'):
-            headers['content-type'] = content_type
-            body = json.dumps(body, indent=indent)
-        return body
-    request.handler = json_handler
+    headers['content-type'] = content_type
+    def handler(*args, **kwargs):
+        body = request._json_inner_handler(*args, **kwargs)
+        return json.dumps(body, indent=indent) if headers['content-type'] == content_type else body
+    request.handler = handler
 
 @tool('on_start_resource')
 def allow(methods=('GET',), paths=()):
@@ -106,41 +112,38 @@ def allow(methods=('GET',), paths=()):
             methods = paths[len(request.handler.args)]
     if 'GET' in methods and 'HEAD' not in methods:
         methods += 'HEAD',
-    if request.method not in methods and not isinstance(request.handler, cherrypy.HTTPError):
-        cherrypy.response.headers['allow'] = ', '.join(methods)
-        message = "The path {0!r} does not allow {1}.".format(request.path_info, request.method)
-        raise cherrypy.HTTPError(httplib.METHOD_NOT_ALLOWED, message)
+    cherrypy.response.headers['allow'] = ', '.join(methods)
+    if request.method not in methods:
+        raise cherrypy.HTTPError(httplib.METHOD_NOT_ALLOWED)
 
 @tool('before_finalize')
-def time_():
+def timer():
     "Return response time in headers."
     response = cherrypy.serving.response
     response.headers['x-response-time'] = time.time() - response.time
 
 @tool('on_start_resource')
-def validate(methods=('GET', 'HEAD'), etag=True, last_modified=True, max_age=None, expires=None):
-    """Return and validate caching headers for GET requests.
+def validate(etag=True, last_modified=True, max_age=None, expires=None):
+    """Return and validate caching headers.
     
-    :param methods: only set headers for specified methods
     :param etag: return weak entity tag header based on index version and validate if-match headers
     :param last_modified: return last-modified header based on index timestamp and validate if-modified headers
     :param max_age: return cache-control max-age and age headers based on last update timestamp
     :param expires: return expires header offset from last update timestamp
     """
-    request = cherrypy.serving.request
+    root = cherrypy.request.app.root
     headers = cherrypy.response.headers
-    if request.method in methods and not isinstance(request.handler, cherrypy.HTTPError):
-        if etag:
-            headers['etag'] = 'W/"{0}"'.format(request.app.root.searcher.version)
-            cherrypy.lib.cptools.validate_etags()
-        if last_modified:
-            headers['last-modified'] = formatdate(request.app.root.searcher.timestamp, usegmt=True)
-            cherrypy.lib.cptools.validate_since()
-        if max_age is not None:
-            headers['age'] = int(time.time() - request.app.root.updated)
-            headers['cache-control'] = 'max-age={0}'.format(max_age)
-        if expires is not None:
-            headers['expires'] = formatdate(expires + request.app.root.updated, usegmt=True)
+    if etag:
+        headers['etag'] = 'W/"{0}"'.format(root.searcher.version)
+        cherrypy.lib.cptools.validate_etags()
+    if last_modified:
+        headers['last-modified'] = formatdate(root.searcher.timestamp, usegmt=True)
+        cherrypy.lib.cptools.validate_since()
+    if max_age is not None:
+        headers['age'] = int(time.time() - root.updated)
+        headers['cache-control'] = 'max-age={0}'.format(max_age)
+    if expires is not None:
+        headers['expires'] = formatdate(expires + root.updated, usegmt=True)
 
 @tool('before_handler')
 def params(**types):
@@ -153,6 +156,12 @@ def params(**types):
 def multi(value):
     return value and value.split(',')
 
+def mapping(value):
+    if isinstance(value, dict):
+        return value
+    cherrypy.response.headers['warning'] = '199 lupyne "use an object instead of an array"'
+    return dict.fromkeys(value, True)
+
 class params:
     "Parameter parsing."
     @staticmethod
@@ -187,7 +196,7 @@ class params:
 
 def json_error(version, **body):
     "Transform errors into json format."
-    tool = cherrypy.request.toolmaps['tools'].get('json', {})
+    tool = cherrypy.request.toolmaps['tools'].get('json_out', {})
     cherrypy.response.headers['content-type'] = tool.get('content_type', 'application/json')
     return json.dumps(body, indent=tool.get('indent'))
 
@@ -208,6 +217,13 @@ class AttachedMonitor(cherrypy.process.plugins.Monitor):
             attach_thread()
             callback()
         cherrypy.process.plugins.Monitor.__init__(self, bus, run, frequency)
+    def subscribe(self):
+        cherrypy.process.plugins.Monitor.subscribe(self)
+        if cherrypy.engine.state == cherrypy.engine.states.STARTED:
+            self.start()
+    def unsubscribe(self):
+        cherrypy.process.plugins.Monitor.unsubscribe(self)
+        self.thread.cancel()
 
 @contextlib.contextmanager
 def HTTPError(status, *exceptions):
@@ -218,10 +234,16 @@ def HTTPError(status, *exceptions):
         raise cherrypy.HTTPError(status, str(exc))
 
 class WebSearcher(object):
-    "Dispatch root with a delegated Searcher."
-    _cp_config = dict.fromkeys(map('tools.{0}.on'.format, ['gzip', 'accept', 'json', 'allow', 'time', 'validate']), True)
+    """Dispatch root with a delegated Searcher.
+    
+    :param hosts: ordered hosts to synchronize with
+    """
+    _cp_config = dict.fromkeys(map('tools.{0}.on'.format, ['gzip', 'accept', 'json_in', 'json_out', 'allow', 'timer', 'validate']), True)
     _cp_config.update({'error_page.default': json_error, 'tools.gzip.mime_types': ['text/html', 'text/plain', 'application/json'], 'tools.accept.media': 'application/json'})
     def __init__(self, *directories, **kwargs):
+        self.hosts = collections.deque(kwargs.pop('hosts', ()))
+        if self.hosts:
+            engine.IndexWriter(*directories).close()
         self.searcher = engine.MultiSearcher(directories, **kwargs) if len(directories) > 1 else engine.IndexSearcher(*directories, **kwargs)
         self.updated = time.time()
     @classmethod
@@ -232,8 +254,40 @@ class WebSearcher(object):
         return self
     def close(self):
         self.searcher.close()
+    def sync(self, host, path=''):
+        "Sync with remote index."
+        path = '/' + '{0}/update/{1}/'.format(path, uuid.uuid1()).lstrip('/')
+        directory = lucene.FSDirectory.cast_(self.searcher.directory).directory.path
+        resource = client.Resource(host)
+        names = sorted(set(resource.put(path)).difference(os.listdir(directory)))
+        try:
+            for name in names:
+                resource.download(path + name, os.path.join(directory, name))
+        finally:
+            resource.delete(path)
+        return names
     @cherrypy.expose
-    @cherrypy.tools.json(process_body=lambda body: dict.fromkeys(body, True))
+    @cherrypy.tools.json_in(process_body=dict)
+    @cherrypy.tools.allow(methods=['GET', 'POST'])
+    def index(self, host='', path=''):
+        """Return index information and synchronize with remote index.
+        
+        **GET, POST** /[index]
+            Return a mapping of the directory to the document count.
+            Add new segments from remote host.
+            
+            {"host": *string*\ [, "path": *string*]}
+            
+            :return: {*string*: *int*,... }
+        """
+        if cherrypy.request.method == 'POST':
+            self.sync(host, path)
+            cherrypy.response.status = httplib.ACCEPTED
+        reader = self.searcher.indexReader
+        readers = reader.sequentialSubReaders if lucene.MultiReader.instance_(reader) else [reader]
+        return dict((unicode(reader.directory()), reader.numDocs()) for reader in readers)
+    @cherrypy.expose
+    @cherrypy.tools.json_in(process_body=mapping)
     @cherrypy.tools.allow(methods=['POST'])
     def update(self, **caches):
         """Refresh index version.
@@ -241,26 +295,34 @@ class WebSearcher(object):
         **POST** /update
             Reopen searcher, optionally reloading caches, and return document count.
             
-            ["filters"|"sorters"|"spellcheckers",... ]
+            {"filters"|"sorters"|"spellcheckers": true,... }
+            
+            .. versionchanged:: 1.2 request body is an object instead of an array
             
             :return: *int*
         """
+        names = ()
+        while self.hosts:
+            host = self.hosts[0]
+            try:
+                names = self.sync(*host.split('/'))
+                break
+            except socket.error:
+                client.Replicas.discard.__func__(self, host)
+            except httplib.HTTPException as exc:
+                assert exc[0] == httplib.METHOD_NOT_ALLOWED, exc
+                break
         self.searcher = self.searcher.reopen(**caches)
         self.updated = time.time()
+        if names:
+            engine.IndexWriter(self.searcher.directory).close()
+        if not self.hosts and hasattr(self, 'fields'):
+            other = WebIndexer(self.searcher.directory, analyzer=self.searcher.analyzer)
+            other.indexer.shared, other.indexer.fields = self.searcher.shared, self.fields
+            app, = (app for app in cherrypy.tree.apps.values() if app.root is self)
+            mount(other, app=app, autoupdate=getattr(self, 'autoupdate', 0))
         return len(self.searcher)
     @cherrypy.expose
-    def index(self):
-        """Return index information.
-        
-        **GET** /
-            Return a mapping of the directory to the document count.
-            
-            :return: {*string*: *int*,... }
-        """
-        reader = self.searcher.indexReader
-        readers = reader.sequentialSubReaders if lucene.MultiReader.instance_(reader) else [reader]
-        return dict((unicode(reader.directory()), reader.numDocs()) for reader in readers)
-    @cherrypy.expose
     @cherrypy.tools.params(**dict.fromkeys(['fields', 'fields.multi', 'fields.indexed', 'fields.vector', 'fields.vector.counts'], multi))
     def docs(self, name=None, value='', **options):
         """Return ids or documents.
@@ -531,49 +593,65 @@ class WebIndexer(WebSearcher):
         else:
             cherrypy.response.status = httplib.ACCEPTED
     @cherrypy.expose
-    @cherrypy.tools.json(process_body=lambda body: {'directories': list(body)})
     @cherrypy.tools.allow(methods=['GET', 'POST'])
-    def index(self, directories=()):
+    def index(self):
         """Add indexes.  See :meth:`WebSearcher.index` for GET method.
         
-        **POST** /
+        **POST** /[index]
             Add indexes without optimization.
             
             [*string*,... ]
         """
-        if cherrypy.request.method == 'POST':
-            for directory in directories:
+        request = cherrypy.serving.request
+        if request.method == 'POST':
+            for directory in getattr(request, 'json', ()):
                 self.indexer += directory
             self.refresh()
         return {unicode(self.indexer.directory): len(self.indexer)}
     @cherrypy.expose
-    @cherrypy.tools.json(process_body=lambda body: dict.fromkeys(body, True))
-    @cherrypy.tools.allow(paths=[('POST',), ('PUT', 'DELETE')])
-    def update(self, id='', **options):
+    @cherrypy.tools.json_in(process_body=mapping)
+    @cherrypy.tools.allow(paths=[('POST',), ('GET', 'PUT', 'DELETE'), ('GET',)])
+    def update(self, id='', name='', **options):
         """Commit index changes and refresh index version.
         
         **POST** /update
             Commit write operations and return document count.  See :meth:`WebSearcher.update` for caching options.
             
-            ["expunge"|"optimize",... ]
+            {"merge": true|\ *int*,... }
+            
+            .. versionchanged:: 1.2 request body is an object instead of an array
             
             :return: *int*
         
-        **PUT, DELETE** /update/*chars*
-            Create unique snapshot of current index commit and return array of referenced filenames.
-            Release previous snapshot by id.
+        **GET, PUT, DELETE** /update/*chars*
+            Verify, create, or release unique snapshot by id of current index commit and return array of referenced filenames.
             
             :return: [*string*,... ]
+        
+        **GET** /update/*chars*/*chars*
+            Download index file corresponding to snapshot id and filename.
         """
         if not id:
             self.indexer.commit(**options)
             self.updated = time.time()
             return len(self.indexer)
-        with HTTPError(httplib.CONFLICT, lucene.JavaError):
-            if cherrypy.request.method == 'PUT':
-                cherrypy.response.status = httplib.CREATED
-                return list(self.indexer.policy.snapshot(id).fileNames)
-            self.indexer.policy.release(id)
+        method = cherrypy.request.method
+        if method == 'DELETE':
+            with HTTPError(httplib.CONFLICT, lucene.JavaError):
+                return self.indexer.policy.release(id)
+        if method == 'PUT':
+            cherrypy.response.status = httplib.CREATED
+            with HTTPError(httplib.CONFLICT, lucene.JavaError):
+                commit = self.indexer.policy.snapshot(id)
+        else:
+            with HTTPError(httplib.NOT_FOUND, lucene.JavaError):
+                commit = self.indexer.policy.getSnapshot(id)
+        if not name:
+            return list(commit.fileNames)
+        with HTTPError(httplib.NOT_FOUND, TypeError, AssertionError):
+            directory = lucene.FSDirectory.cast_(self.indexer.directory).directory.path
+            assert name in commit.fileNames, 'file not referenced in commit'
+        return cherrypy.lib.static.serve_download(os.path.join(directory, name))
     @cherrypy.expose
     @cherrypy.tools.allow(paths=[('GET', 'POST'), ('GET',), ('GET', 'PUT', 'DELETE')])
     def docs(self, name=None, value='', **options):
@@ -623,7 +701,7 @@ class WebIndexer(WebSearcher):
         self.refresh()
     search._cp_config.update(WebSearcher.search._cp_config)
     @cherrypy.expose
-    @cherrypy.tools.json(process_body=dict)
+    @cherrypy.tools.json_in(process_body=dict)
     @cherrypy.tools.allow(paths=[('GET',), ('GET', 'PUT')])
     @cherrypy.tools.validate(on=False)
     def fields(self, name='', **settings):
@@ -659,17 +737,25 @@ def init(vmargs='-Xrs', **kwargs):
         if isinstance(app.root, WebSearcher):
             app.root.__init__(*app.root.__dict__.pop('args'), **app.root.__dict__.pop('kwargs'))
 
-def mount(root, path='', config=None, autoupdate=0):
+def mount(root, path='', config=None, autoupdate=0, app=None):
     """Attach root and subscribe to plugins.
     
     :param root,path,config: see cherrypy.tree.mount
     :param autoupdate: see command-line options
+    :param app: optionally replace root on existing app
     """
-    if hasattr(root, 'close'):
-        cherrypy.engine.subscribe('stop', root.close)
+    if app is None:
+        app = cherrypy.tree.mount(root, path, config)
+    else:
+        cherrypy.engine.unsubscribe('stop', app.root.close)
+        if hasattr(app.root, 'monitor'):
+            app.root.monitor.unsubscribe()
+        app.root = root
+    cherrypy.engine.subscribe('stop', root.close)
     if autoupdate:
-        AttachedMonitor(cherrypy.engine, root.update, autoupdate).subscribe()
-    return cherrypy.tree.mount(root, path, config)
+        root.monitor = AttachedMonitor(cherrypy.engine, root.update, autoupdate)
+        root.monitor.subscribe()
+    return app
 
 def start(root=None, path='', config=None, pidfile='', daemonize=False, autoreload=0, autoupdate=0, callback=None):
     """Attach root, subscribe to plugins, and start server.
@@ -701,16 +787,22 @@ parser.add_option('-p', '--pidfile', metavar='FILE', help='store the process id
 parser.add_option('-d', '--daemonize', action='store_true', help='run the server as a daemon')
 parser.add_option('--autoreload', type=float, metavar='SECONDS', help='automatically reload modules; replacement for engine.autoreload')
 parser.add_option('--autoupdate', type=float, metavar='SECONDS', help='automatically update index version and commit any changes')
+parser.add_option('--autosync', metavar='HOST[:PORT][/PATH],...', help='automatically synchronize searcher with remote hosts and update')
 parser.add_option('--real-time', action='store_true', help='search in real-time without committing')
 
 if __name__ == '__main__':
     options, args = parser.parse_args()
-    read_only = options.__dict__.pop('read_only') or len(args) > 1
-    kwargs = {'nrt': True} if options.__dict__.pop('real_time') else {}
-    if read_only and (kwargs or not args):
+    read_only = options.read_only or options.autosync or len(args) > 1
+    kwargs = {'nrt': True} if options.real_time else {}
+    if read_only and (options.real_time or not args):
         parser.error('incompatible read/write options')
+    if options.autosync:
+        kwargs['hosts'] = options.autosync.split(',')
+        if not (options.autoupdate and len(args) == 1):
+            parser.error('autosync requires autoupdate and a single directory')
     if options.config and not os.path.exists(options.config):
         options.config = {'global': json.loads(options.config)}
     cls = WebSearcher if read_only else WebIndexer
     root = cls.new(*map(os.path.abspath, args), **kwargs)
+    del options.read_only, options.autosync, options.real_time
     start(root, callback=init, **options.__dict__)
