@@ -1,10 +1,13 @@
 from future_builtins import map
 import unittest
+import os
+import sys, subprocess
 import heapq
 import time
 import socket, httplib
-from lupyne import client
-from . import local, remote
+import lucene, cherrypy
+from lupyne import client, server
+from . import remote
 
 def getresponse(error):
     "Test error handling in resources."
@@ -13,12 +16,10 @@ def getresponse(error):
 class TestCase(remote.BaseTest):
     ports = 8080, 8081, 8082
     hosts = list(map('localhost:{0:d}'.format, ports))
-    def setUp(self):
-        local.BaseTest.setUp(self)
-        self.servers = list(map(self.start, self.ports))
     
     def testInterface(self):
         "Distributed reading and writing."
+        self.servers += map(self.start, self.ports)
         resources = client.Resources(self.hosts, limit=1)
         assert resources.unicast('GET', '/')
         assert not resources.unicast('POST', '/terms')
@@ -59,17 +60,20 @@ class TestCase(remote.BaseTest):
         assert resources.broadcast('GET', '/')
         assert list(map(len, resources.values())) == counts[::-1]
         host = self.hosts[1]
-        resource = resources.request(host, 'GET', '/')
+        stream = resources[host].stream('GET', '/')
+        resource = next(stream)
         resource.getresponse = lambda: getresponse(socket.error)
-        self.assertRaises(socket.error, resources.getresponse, host, resource)
-        resource = resources.request(host, 'GET', '/')
+        self.assertRaises(socket.error, next, stream)
+        stream = resources[host].stream('GET', '/')
+        resource = next(stream)
         resource.getresponse = lambda: getresponse(httplib.BadStatusLine)
-        assert resources.getresponse(host, resource) is None
+        assert next(stream) is None
         resources.clear()
         self.assertRaises(ValueError, resources.unicast, 'GET', '/')
     
     def testSharding(self):
         "Sharding of indices across servers."
+        self.servers += map(self.start, self.ports)
         keys = range(len(self.hosts))
         shards = client.Shards(zip(self.hosts * 2, heapq.merge(keys, keys)), limit=1)
         shards.resources.broadcast('PUT', '/fields/zone', {'store': 'yes'})
@@ -97,6 +101,63 @@ class TestCase(remote.BaseTest):
         assert len(responses) == 2 and all(response() for response in responses)
         shards.resources.priority = lambda hosts: None
         self.assertRaises(ValueError, shards.choice, [[0]])
+    
+    def testReplication(self):
+        "Replication from indexer to searcher."
+        directory = os.path.join(self.tempdir, 'backup')
+        sync, update = '--autosync=' + self.hosts[0], '--autoupdate=1'
+        self.servers += (
+            self.start(self.ports[0], self.tempdir),
+            self.start(self.ports[1], '-r', directory, sync, update),
+            self.start(self.ports[2], '-r', directory),
+        )
+        for args in [('-r', self.tempdir), (update, self.tempdir), (update, self.tempdir, self.tempdir)]:
+            assert subprocess.call((sys.executable, '-m', 'lupyne.server', sync) + args, stderr=subprocess.PIPE)
+        replicas = client.Replicas(self.hosts[:2], limit=1)
+        replicas.discard(None)
+        replicas.post('/docs', [{}])
+        assert replicas.post('/update') == 1
+        resource = client.Resource(self.hosts[2])
+        response = resource.call('POST', '/', {'host': self.hosts[0]})
+        assert response.status == httplib.ACCEPTED and sum(response().values()) == 0
+        assert resource.post('/update') == 1
+        assert resource.post('/', {'host': self.hosts[0], 'path': '/'})
+        assert resource.post('/update') == 1
+        replicas.post('/docs', [{}])
+        assert replicas.post('/update') == 2
+        resource = client.Resource(self.hosts[1])
+        time.sleep(1.1)
+        assert sum(resource.get('/').values()) == 2
+        self.stop(self.servers.pop())
+        root = server.WebSearcher(directory, hosts=self.hosts[:2])
+        app = server.mount(root)
+        root.fields = {}
+        assert root.update() == 2
+        assert len(root.hosts) == 2
+        self.stop(self.servers.pop(0))
+        assert replicas.get('/docs')
+        assert replicas.call('POST', '/docs', []).status == httplib.METHOD_NOT_ALLOWED
+        assert replicas.get('/terms', option='indexed') == []
+        assert replicas.call('POST', '/docs', [], retry=True).status == httplib.METHOD_NOT_ALLOWED
+        assert root.update() == 2
+        assert len(root.hosts) == 1
+        self.stop(self.servers.pop(0))
+        assert root.update() == 2
+        assert len(root.hosts) == 0 and isinstance(app.root, server.WebIndexer)
+        app.root.close()
+        root = server.WebSearcher(directory)
+        app = server.mount(root, autoupdate=0.1)
+        root.fields, root.autoupdate = {}, 0.1
+        cherrypy.config['log.screen'] = self.config['log.screen']
+        cherrypy.engine.state = cherrypy.engine.states.STARTED
+        root.monitor.start() # simulate engine starting
+        time.sleep(0.2)
+        app.root.indexer.add()
+        time.sleep(0.2)
+        assert len(app.root.indexer) == len(root.searcher) + 1
+        app.root.monitor.unsubscribe()
+        del app.root
 
 if __name__ == '__main__':
+    lucene.initVM()
     unittest.main()
