@@ -15,17 +15,14 @@ from . import fixture
 
 class typeAsPayload(engine.TokenFilter):
     "Custom implementation of lucene TypeAsPayloadTokenFilter."
-    def incrementToken(self):
-        result = engine.TokenFilter.incrementToken(self)
+    def setattrs(self):
         self.payload = self.type.encode('utf8')
-        return result
 
 @contextlib.contextmanager
 def assertWarns(*categories):
     with warnings.catch_warnings(record=True) as messages:
         yield
-    assert len(messages) == len(categories), messages
-    for message, category in zip(messages, categories):
+    for message, category in itertools.izip_longest(messages, categories):
          assert issubclass(message.category, category), message
 
 class Filter(lucene.PythonFilter):
@@ -56,7 +53,8 @@ class TestCase(BaseTest):
             token.offset, token.positionIncrement = (0, 0), 0
         assert str(stemmer.parse('hellos', field=['body', 'title'])) == 'body:hello title:hello'
         assert str(stemmer.parse('hellos', field={'body': 1.0, 'title': 2.0})) == 'body:hello title:hello^2.0'
-        indexer = engine.Indexer(analyzer=stemmer, version=lucene.Version.LUCENE_30)
+        indexer = engine.Indexer(analyzer=stemmer, version=lucene.Version.LUCENE_30, writeLockTimeout=100L)
+        assert indexer.writeLockTimeout == 100
         self.assertRaises(lucene.JavaError, engine.Indexer, indexer.directory)
         indexer.set('text')
         indexer.set('name', store=True, index=False, boost=2.0)
@@ -66,6 +64,7 @@ class TestCase(BaseTest):
         searcher = indexer.indexSearcher
         indexer.commit()
         assert searcher is indexer.indexSearcher
+        assert not searcher.search(count=1)
         indexer.add(text='hello worlds', name='sample', tag=['python', 'search'])
         assert len(indexer) == 1 and list(indexer) == []
         assert not indexer.optimized
@@ -85,8 +84,8 @@ class TestCase(BaseTest):
         assert not list(indexer.termvector(0, 'tag'))
         assert indexer.count('text', 'hello') == indexer.count('text:hello') == 1
         assert sorted(indexer.names()) == ['name', 'tag', 'text']
-        assert sorted(indexer.names('indexed')) == ['tag', 'text']
-        assert indexer.names('unindexed') == ['name']
+        assert sorted(indexer.names('indexed', isIndexed=True)) == ['tag', 'text']
+        assert indexer.names('unindexed', isIndexed=False) == ['name']
         assert list(indexer.terms('text')) == ['hello', 'world']
         assert list(indexer.terms('text', 'h', 'v')) == ['hello']
         assert dict(indexer.terms('text', 'w', counts=True)) == {'world': 1}
@@ -108,7 +107,7 @@ class TestCase(BaseTest):
         assert indexer.search('text:hello hi') and not indexer.search('text:hello hi', op='and')
         assert indexer.search('text:*hello', allowLeadingWildcard=True)
         query = engine.Query.multiphrase('text', ('hello', 'hi'), None, 'world')
-        assert str(query) == 'text:"(hello hi) world"' and list(query.positions) == [0, 2]
+        assert str(query).startswith('text:"(hello hi) ') and list(query.positions) == [0, 2]
         query = engine.Query.wildcard('text', '*')
         assert str(query) == 'text:*' and isinstance(query, lucene.WildcardQuery)
         assert str(lucene.MatchAllDocsQuery() | query) == '*:* text:*'
@@ -120,6 +119,7 @@ class TestCase(BaseTest):
         query = engine.Query.span('text', 'world')
         assert str(query.mask('name')) == 'mask(text:world) as name'
         assert str(query.payload()) == 'spanPayCheck(text:world, payloadRef: )'
+        assert isinstance(query.filter(cache=False), lucene.SpanQueryFilter) and isinstance(query.filter(), lucene.CachingSpanFilter)
         query = engine.Query.disjunct(0.1, query, name='sample')
         assert str(query) == '(text:world | name:sample)~0.1'
         query = engine.Query.near('text', 'hello', ('tag', 'python'), slop=-1, inOrder=False)
@@ -162,10 +162,13 @@ class TestCase(BaseTest):
         assert reader[0].dict() == {} and reader.count('text', '?') == 1
         assert len(reader.comparator('text')) == 4
         indexer.delete('text', '?')
-        indexer.commit(expunge=True)
+        with assertWarns(DeprecationWarning):
+            indexer.commit(expunge=True)
+        indexer.commit(merge=True)
         assert not indexer.hasDeletions()
-        indexer.commit(optimize=2)
-        indexer.commit(optimize=True)
+        indexer.commit(merge=2)
+        with assertWarns(DeprecationWarning):
+            indexer.commit(optimize=True)
         assert indexer.optimized
         del reader.indexReader
         self.assertRaises(AttributeError, getattr, reader, 'maxDoc')
@@ -190,13 +193,16 @@ class TestCase(BaseTest):
         assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
         indexer.delete('amendment', doc['amendment'])
         indexer.add(doc)
+        reader = indexer.indexReader
         indexer.commit(filters=True, spellcheckers=True)
+        assert reader.refCount == 0
         assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
         doc['amendment'] = engine.Analyzer(lucene.WhitespaceTokenizer).tokens(doc['amendment'])
         doc['date'] = engine.Analyzer(lucene.WhitespaceTokenizer).tokens(doc['date']), 2.0
         scores = list(searcher.match(doc, 'text:congress', 'text:law', 'amendment:27', 'date:19*'))
         assert 0.0 == scores[0] < scores[1] < scores[2] < scores[3] == 1.0
         searcher = engine.MultiSearcher([indexer.indexReader, self.tempdir])
+        assert searcher.refCount == 1
         assert searcher.count() == len(searcher) == 2 * len(indexer)
         searcher.sorters['amendment'] = engine.SortField('amenmdment', int)
         comparator = searcher.comparator('amendment')
@@ -264,7 +270,7 @@ class TestCase(BaseTest):
         query = engine.Query.term('text', 'right', boost=2.0)
         assert query.boost == 2.0
         assert indexer.facets(str(query), 'amendment', 'article') == {'amendment': 12, 'article': 1}
-        self.assertRaises(TypeError, indexer.overlap, query.filter(), query.filter(cache=False))
+        self.assertRaises(TypeError, indexer.overlap, query.filter(), lucene.QueryWrapperFilter(query))
         hits = indexer.search('text:people', filter=query.filter())
         assert len(hits) == 4
         hit, = indexer.search('date:192*')
@@ -273,7 +279,7 @@ class TestCase(BaseTest):
         amendments = ['18', '19']
         assert sorted(hit['amendment'] for hit in hits) == amendments
         query = engine.Query.range('date', '1919', '1921')
-        hits = indexer.search(query)
+        hits = indexer.search(filter=query.filter())
         assert sorted(hit['amendment'] for hit in hits) == amendments
         hits = indexer.search(query | engine.Query.term('text', 'vote'))
         assert set(hit.get('amendment') for hit in hits) > set(amendments)
@@ -344,7 +350,7 @@ class TestCase(BaseTest):
         files = set(os.listdir(self.tempdir))
         path = os.path.join(self.tempdir, 'temp')
         with indexer.snapshot('backup') as commit:
-            indexer.commit(optimize=True)
+            indexer.commit(merge=1)
             assert indexer.indexCommit.generation > commit.generation
             engine.indexers.copy(commit, path)
             assert set(os.listdir(path)) == set(commit.fileNames) < files < set(os.listdir(self.tempdir))
@@ -368,8 +374,8 @@ class TestCase(BaseTest):
                 location = '.'.join(doc[name] for name in ['state', 'county', 'city'])
                 indexer.add(doc, latitude=lat, longitude=lng, location=location)
         indexer.commit()
-        assert set(['state', 'zipcode']) < set(indexer.names('indexed'))
-        assert set(['latitude', 'longitude', 'county', 'city']) == set(indexer.names('unindexed'))
+        assert set(['state', 'zipcode']) < set(indexer.names('indexed', isIndexed=True))
+        assert set(['latitude', 'longitude', 'county', 'city']) == set(indexer.names('unindexed', isIndexed=False))
         states = list(indexer.terms('state'))
         assert states[0] == 'AK' and states[-1] == 'WY'
         counties = [term.split('.')[-1] for term in indexer.terms('state.county', 'CA', 'CA~')]
@@ -403,7 +409,8 @@ class TestCase(BaseTest):
         searcher = engine.IndexSearcher(directory)
         assert len(searcher) == size and list(searcher.terms('state')) == ['CA']
         path = os.path.join(self.tempdir, 'temp')
-        size = indexer.copy(path, exclude=query, optimize=True)
+        with assertWarns(DeprecationWarning):
+            size = indexer.copy(path, exclude=query, optimize=True)
         assert len(searcher) + size == len(indexer)
         searcher = engine.IndexSearcher(path)
         assert searcher.optimized and 'CA' not in searcher.terms('state')
@@ -457,8 +464,13 @@ class TestCase(BaseTest):
         assert 1 == counts[0] < counts[2] < counts[1]
         assert len(field.within(x, y, 10**8)) == 1
         self.assertRaises(NameError, list, field.radiate(y, x, 1, 0))
+        hits = hits.filter(lambda id: distances[id] < 10**4)
+        assert 0 < len(hits) < sum(counts.values())
+        hits = hits.sorted(distances.__getitem__, reverse=True)
+        assert 0 == distances[hits.ids[-1]] < distances[hits.ids[0]] < 10**4
         if hasattr(lucene, 'LatLongDistanceFilter'):
-            f = field.filter(x, y, 10**4, 'longitude', 'latitude')
+            with assertWarns(DeprecationWarning):
+                f = field.filter(x, y, 10**4, 'longitude', 'latitude')
             ids = indexer.search(query, sort=distances.__getitem__).ids
             hits = indexer.search(count=10, filter=f, sort=f.sorter())
             assert len(hits) < len(ids) and hits.ids == ids[:len(hits)]
@@ -513,6 +525,7 @@ class TestCase(BaseTest):
         assert [int(hit['amendment']) for hit in hits] == [1, 2, 3]
         hits = indexer.search(count=3, sort='year', reverse=True)
         assert [int(hit['amendment']) for hit in hits] == [27, 26, 25]
+        assert indexer.count(filter=indexer.sorters['year'].filter(None, 1792)) == 10
         assert cache == len(lucene.FieldCache.DEFAULT.cacheEntries)
         indexer.add()
         indexer.commit(sorters=True)
@@ -530,8 +543,7 @@ class TestCase(BaseTest):
             if 'amendment' in doc:
                 indexer.add(amendment=int(doc['amendment']), date=[tuple(map(int, doc['date'].split('-')))], size=len(doc['text']))
         indexer.commit()
-        query = indexer.fields['amendment'].range(None, 10)
-        assert indexer.count(query) == 9
+        assert indexer.count(filter=indexer.fields['amendment'].filter(None, 10)) == 9
         field = indexer.fields['date']
         query = field.prefix((1791, 12))
         assert indexer.count(query) == 10
@@ -614,6 +626,40 @@ class TestCase(BaseTest):
         indexer.add()
         indexer.commit()
         assert indexer.count() == engine.IndexSearcher(indexer.directory).count() == 2
+    
+    def testFilters(self):
+        "Custom filters."
+        if not hasattr(lucene, 'FixedBitSet'):
+            return self.assertRaises(AssertionError, engine.queries.TermsFilter, '')
+        indexer = engine.Indexer()
+        indexer.set('name', store=True, index=True)
+        for name in ('alpha', 'bravo', 'charlie'):
+            indexer.add(name=name)
+        indexer.commit()
+        filter = engine.TermsFilter('name')
+        assert indexer.count(filter=filter) == len(filter.readers) == 0
+        filter.add('alpha', 'bravo')
+        filter.discard('bravo', 'charlie')
+        assert filter.values == set(['alpha'])
+        parallel = engine.ParallelIndexer('name')
+        parallel.set('priority', index=True)
+        for name in ('alpha', 'bravo', 'delta'):
+            parallel.update(name, priority='high')
+        parallel.commit()
+        filter = parallel.termsfilter(engine.Query.term('priority', 'high').filter(), indexer)
+        assert [hit['name'] for hit in indexer.search(filter=filter)] == ['alpha', 'bravo']
+        indexer.add(name='delta')
+        indexer.delete('name', 'alpha')
+        indexer.commit()
+        assert filter.readers > set(indexer.sequentialSubReaders)
+        assert [hit['name'] for hit in indexer.search(filter=filter)] == ['bravo', 'delta']
+        parallel.update('bravo')
+        parallel.update('charlie', priority='high')
+        parallel.commit()
+        assert [hit['name'] for hit in indexer.search(filter=filter)] == ['charlie', 'delta']
+        parallel.commit()
+        filter.refresh(indexer)
+        assert filter.readers == set(indexer.sequentialSubReaders)
 
 if __name__ == '__main__':
     lucene.initVM()
