@@ -66,14 +66,15 @@ class TestCase(BaseTest):
     
     def testInterface(self):
         "Remote reading and writing."
+        config = {'tools.json_out.indent': 2, 'tools.validate.expires': 0, 'tools.validate.max_age': 0}
         self.servers += (
-            self.start(self.ports[0], self.tempdir, '--autoreload=1', **{'tools.validate.expires': 0, 'tools.validate.max_age': 0}),
+            self.start(self.ports[0], self.tempdir, '--autoreload=1', **config),
             self.start(self.ports[1], self.tempdir, self.tempdir, '--autoupdate=2.0'), # concurrent searchers
         )
         resource = client.Resource('localhost', self.ports[0])
         assert resource.get('/favicon.ico')
         response = resource.call('GET', '/')
-        assert response.status == httplib.OK and response.reason == 'OK' and response.time > 0
+        assert response.status == httplib.OK and response.reason == 'OK' and response.time > 0 and '\n' in response.body
         assert response.getheader('content-encoding') == 'gzip' and response.getheader('content-type').startswith('application/json')
         version, modified = response.getheader('etag'), response.getheader('last-modified')
         assert version.strip('W/"').isdigit()
@@ -81,12 +82,14 @@ class TestCase(BaseTest):
         dates = list(map(parsedate, [modified, response.getheader('expires'), response.getheader('date')]))
         assert all(dates) and sorted(dates) == dates
         (directory, count), = response().items()
-        assert count == 0 and 'FSDirectory@' in directory
+        assert count == 0 and 'Directory@' in directory
         assert resource.call('HEAD', '/').status == httplib.OK
         with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
             resource.post('/terms')
         with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
             resource.get('/update')
+        with local.assertWarns(UserWarning):
+            resource.post('/update', [])
         with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
             resource.post('/update/snapshot')
         with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
@@ -152,19 +155,21 @@ class TestCase(BaseTest):
         del resource.headers['if-none-match']
         resource.headers['if-modified-since'] = modified
         assert resource.call('GET', '/').status == httplib.NOT_MODIFIED
+        del resource.headers['if-modified-since']
         time.sleep(max(0, calendar.timegm(parsedate(modified)) + 1 - time.time()))
         assert resource.post('/update')
         response = resource.call('GET', '/')
         assert response and response.getheader('etag') != version and parsedate(response.getheader('last-modified')) > parsedate(modified)
         resource.headers['if-match'] = version
-        assert resource.call('GET', '/docs/0').status == httplib.PRECONDITION_FAILED
+        response = resource.call('GET', '/docs/0')
+        assert response.status == httplib.PRECONDITION_FAILED and '\n' in response.body
         del resource.headers['if-match']
         assert resource.get('/docs') == [0]
         assert resource.get('/docs/0') == resource.get('/docs/name/sample') == {'name': 'sample'}
         assert resource.get('/docs/0', fields='missing') == {'missing': None}
         assert resource.get('/docs/0', fields='', **{'fields.multi': 'missing'}) == {'missing': []}
         assert resource.get('/terms') == ['name', 'text']
-        assert resource.get('/terms', option='unindexed') == []
+        assert resource.get('/terms', option='unindexed', isIndexed=False) == []
         assert resource.get('/terms/text') == ['hello', 'world']
         assert resource.get('/terms/text/world') == 1
         with assertRaises(httplib.HTTPException, httplib.BAD_REQUEST):
@@ -214,7 +219,7 @@ class TestCase(BaseTest):
         assert result['facets'] == {'name': {'sample': 1}} and result['spellcheck'] == {}
         resource = client.Resource('localhost', self.ports[-1])
         assert set(resource.get('/').values()) < set([0, 1])
-        assert resource.post('/update', ['filters', 'sorters']) == 2
+        assert resource.post('/update', {'filters': True, 'sorters': True}) == 2
         assert resource.get('/docs') == [0, 1]
         with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
             resource.get('/docs/name/sample')
@@ -223,7 +228,7 @@ class TestCase(BaseTest):
         resource = client.Resource('localhost', self.ports[0])
         assert not resource.delete('/search', q='sample', **{'q.field': 'name', 'q.type': 'term'})
         assert resource.get('/docs') == [0]
-        assert not resource.post('/update', ['expunge'])
+        assert not resource.post('/update', {'merge': True})
         assert resource.get('/docs') == []
         assert not resource.put('/docs/name/sample')
         assert resource.post('/update')
@@ -245,15 +250,27 @@ class TestCase(BaseTest):
         with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
             resource.get('/docs/missing/sample')
         assert not resource.delete('/search')
-        responses = resource.multicall(('POST', '/docs', [{}]), ('POST', '/update'), ('GET', '/docs'))
+        responses = list(resource.multicall(('POST', '/docs', [{}]), ('POST', '/update'), ('GET', '/docs')))
         assert responses[0].status == httplib.ACCEPTED and responses[1]() == len(responses[2]()) == 1
         assert resource.post('/', [self.tempdir]).values() == [2]
         with local.assertWarns(DeprecationWarning, UserWarning):
             assert Resource(resource.host, resource.port).call('GET', '/missing', redirect=True)()
+        with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
+            resource.get('/update/snapshot')
         response = resource.call('PUT', '/update/snapshot')
         assert response.status == httplib.CREATED
-        assert all(name.startswith('_') or name.startswith('segments_') for name in response())
-        assert resource.put('/update/backup') == response()
+        names = response()
+        assert all(name.startswith('_') or name.startswith('segments_') for name in names)
+        assert resource.get('/update/snapshot') == names
+        with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
+            resource.get('/update/snapshot/segments')
+        with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
+            resource.download('/update/snapshot/segments.gen', self.tempdir)
+        for name in names:
+            response = resource.call('GET', '/update/snapshot/' + name)
+            assert response and response.getheader('content-type') == 'application/x-download'
+            assert len(response.body) == os.path.getsize(os.path.join(self.tempdir, name))
+        assert resource.put('/update/backup') == names
         with assertRaises(httplib.HTTPException, httplib.CONFLICT):
             resource.put('/update/snapshot')
         assert not resource.delete('/update/snapshot')
@@ -269,7 +286,7 @@ class TestCase(BaseTest):
         os.kill(int(open(pidfile).read()), signal.SIGTERM)
         filepath = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'lupyne/server.py')
         assert subprocess.call((sys.executable, filepath, '-c', filepath), stderr=subprocess.PIPE)
-        assert server.mount(None, '/path', config={'': {}})
+        assert cherrypy.tree.mount(None)
         server.init(vmargs=None)
         self.assertRaises(AttributeError, server.start, config=True)
     
@@ -286,7 +303,7 @@ class TestCase(BaseTest):
             assert sorted(resource.get('/fields/' + name)) == ['index', 'store', 'termvector']
         resource.post('/docs', list(fixture.constitution.docs()))
         assert resource.get('/').values() == [35]
-        resource.post('/update', ['optimize', 'spellcheckers'])
+        resource.post('/update', {'spellcheckers': True, 'merge': 1})
         assert resource.get('/docs/0', **{'fields.indexed': 'amendment:int'}) == {'amendment': 0, 'article': 'Preamble'}
         doc = resource.get('/docs/0', **{'fields.vector': 'text,missing'})
         assert doc['missing'] == [] and doc['text'].index('states') < doc['text'].index('united')
